/*
	This file is part of lgui.

	lgui is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lgui is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lgui.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/

#include "action.h"

int lgui_action_new(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TSTRING);
	int top = lua_gettop(L);
	
	const gchar* str[4] = {NULL, NULL, NULL, NULL};
	str[0] = lua_tostring(L, 1);
	int i;
	
	for(i = 2; i <= top; ++i)
	{
		if(lua_type(L, i) == LUA_TSTRING)
			str[i - 1] = lua_tostring(L, i);
	}
	
	GtkAction* action = gtk_action_new(str[0], str[1], str[2], str[3]);
	object_new(L, action, TRUE);
	
	return 1;
}

int lgui_action_activate(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	gtk_action_activate(GTK_ACTION(obj->pointer));
	
	return 0;
}

int lgui_action_create_icon(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);
	Object* obj = lua_touserdata(L, 1);
	
	GtkWidget* ptr = gtk_action_create_icon(GTK_ACTION(obj->pointer), lua_tointeger(L, 2));
	object_new(L, ptr, FALSE);
	
	return 1;
}

int lgui_action_create_menu_item(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	GtkWidget* ptr = gtk_action_create_menu_item(GTK_ACTION(obj->pointer));
	object_new(L, ptr, FALSE);
	
	return 1;
}

int lgui_action_create_tool_item(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	GtkWidget* ptr = gtk_action_create_tool_item(GTK_ACTION(obj->pointer));
	object_new(L, ptr, FALSE);
	
	return 1;
}

int lgui_action_connect_proxy(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 2);
	
	gtk_action_connect_proxy(GTK_ACTION(obj1->pointer), GTK_WIDGET(obj2->pointer));
	
	return 0;
}

int lgui_action_disconnect_proxy(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 2);
	
	gtk_action_disconnect_proxy(GTK_ACTION(obj1->pointer), GTK_WIDGET(obj2->pointer));
	
	return 0;
}

int lgui_action_get_proxies(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	GSList* proxies = gtk_action_get_proxies(GTK_ACTION(obj->pointer));
	lua_pushstring(L, "__lobjGSList2table");
	lua_rawget(L, LUA_GLOBALSINDEX);
	lua_pushlightuserdata(L, proxies);
	lua_pushinteger(L, TYPE_OBJECT);
	lua_call(L, 2, 1);
	
	return 1;
}

int lgui_action_set_accel_group(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	
	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 2);
	gtk_action_set_accel_group(GTK_ACTION(obj1->pointer), GTK_ACCEL_GROUP(obj2->pointer));
	
	return 0;
}

int lgui_action_connect_accelerator(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	gtk_action_connect_accelerator(GTK_ACTION(obj->pointer));
	
	return 0;
}

int lgui_action_disconnect_accelerator(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	gtk_action_disconnect_accelerator(GTK_ACTION(obj->pointer));
	
	return 0;
}

/* GtkRecentAction */

int lgui_recent_action_new(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TSTRING);
	int top = lua_gettop(L);
	
	const gchar* str[4] = {NULL, NULL, NULL, NULL};
	str[0] = lua_tostring(L, 1);
	int i;
	
	for(i = 2; i <= top; ++i)
	{
		if(lua_type(L, i) == LUA_TSTRING)
			str[i - 1] = lua_tostring(L, i);
	}
	
	GtkAction* action = gtk_recent_action_new(str[0], str[1], str[2], str[3]);
	object_new(L, action, TRUE);
	
	return 1;
}

int lgui_recent_action_new_for_manager(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TSTRING);
	int top = lua_gettop(L);
	
	const gchar* str[4] = {NULL, NULL, NULL, NULL};
	str[0] = lua_tostring(L, 1);
	int i;
	
	for(i = 2; i <= top && i < 5; ++i)
	{
		if(lua_type(L, i) == LUA_TSTRING)
			str[i - 1] = lua_tostring(L, i);
	}
	
	GtkRecentManager* manager = NULL;
	
	if(top > 4)
	{
		luaL_checktype(L, 5, LUA_TUSERDATA);
		Object* obj2 = lua_touserdata(L, 5);
		manager = GTK_RECENT_MANAGER(obj2->pointer);
	}
	
	GtkAction* action = gtk_recent_action_new_for_manager(str[0], str[1], str[2], str[3], manager);
	object_new(L, action, TRUE);
	
	return 1;
}

/* GtkToggleAction */

int lgui_toggle_action_new(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TSTRING);
	int top = lua_gettop(L);
	
	const gchar* str[4] = {NULL, NULL, NULL, NULL};
	str[0] = lua_tostring(L, 1);
	int i;
	
	for(i = 2; i <= top; ++i)
	{
		if(lua_type(L, i) == LUA_TSTRING)
			str[i - 1] = lua_tostring(L, i);
	}
	
	GtkToggleAction* action = gtk_toggle_action_new(str[0], str[1], str[2], str[3]);
	object_new(L, action, TRUE);
	
	return 1;
}

int lgui_toggle_action_toggled(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	gtk_toggle_action_toggled(GTK_TOGGLE_ACTION(obj->pointer));
	
	return 0;
}

/* GtkRadioAction */

int lgui_radio_action_new(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TSTRING);
	int top = lua_gettop(L);
	
	const gchar* str[4] = {NULL, NULL, NULL, NULL};
	str[0] = lua_tostring(L, 1);
	int i;
	
	for(i = 2; i <= top && i < 5; ++i)
	{
		if(lua_type(L, i) == LUA_TSTRING)
			str[i - 1] = lua_tostring(L, i);
	}
	
	gint value = 0;
	
	if(top > 4)
	{
		luaL_checktype(L, 5, LUA_TNUMBER);
		value = lua_tointeger(L, 5);
	}
	
	GtkRadioAction* action = gtk_radio_action_new(str[0], str[1], str[2], str[3], value);
	object_new(L, action, TRUE);
	
	return 1;
}

/* GtkActionGroup */

int lgui_action_group_new(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TSTRING);
	GtkActionGroup* ptr = gtk_action_group_new(lua_tostring(L, 1));
	object_new(L, ptr, TRUE);
	
	return 1;
}

int lgui_action_group_add_action(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	int top = lua_gettop(L), i;
	
	Object* obj1 = lua_touserdata(L, 1);
	
	for(i = 2; i <= top; ++i)
	{
		luaL_checktype(L, i, LUA_TUSERDATA);
		Object* obj2 = lua_touserdata(L, i);
		gtk_action_group_add_action(GTK_ACTION_GROUP(obj1->pointer), GTK_ACTION(obj2->pointer));	
	}
	
	return 0;
}

int lgui_action_group_add_action_with_accel(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	
	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 2);
	int top = lua_gettop(L);
	gchar* str = NULL;
	
	if(top > 2)
		str = (gchar*)lua_tostring(L, 3);
	
	gtk_action_group_add_action_with_accel(GTK_ACTION_GROUP(obj1->pointer),
		GTK_ACTION(obj2->pointer), str);	
	
	return 0;
}

int lgui_action_group_get_action(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);
	
	Object* obj = lua_touserdata(L, 1);
	
	GtkAction* ptr = gtk_action_group_get_action(GTK_ACTION_GROUP(obj->pointer),
		lua_tostring(L, 2));
		
	object_new(L, ptr, FALSE);
	
	return 1;
}

int lgui_action_group_remove_action(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	
	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 2);
	gtk_action_group_remove_action(GTK_ACTION_GROUP(obj1->pointer), GTK_ACTION(obj2->pointer));
	
	return 0;
}

int lgui_action_group_list_actions(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	GList* actions = gtk_action_group_list_actions(GTK_ACTION_GROUP(obj->pointer));
	lua_pushstring(L, "__lobjGList2table");
	lua_rawget(L, LUA_GLOBALSINDEX);
	lua_pushlightuserdata(L, actions);
	lua_pushinteger(L, TYPE_OBJECT);
	lua_call(L, 2, 1);

	g_list_free(actions);
	
	return 1;
}
