/*
	This file is part of lgui.

	lgui is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lgui is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lgui.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/

#include "box.h"

/* Box */

int lgui_hbox_new(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TBOOLEAN);
	luaL_checktype(L, 2, LUA_TNUMBER);
	
	GtkWidget* ptr = gtk_hbox_new(lua_toboolean(L, 1), lua_tointeger(L, 2));

	object_new(L, ptr, FALSE);
	return 1;
}

int lgui_vbox_new(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TBOOLEAN);
	luaL_checktype(L, 2, LUA_TNUMBER);
	
	GtkWidget* ptr = gtk_vbox_new(lua_toboolean(L, 1), lua_tointeger(L, 2));

	object_new(L, ptr, FALSE);
	return 1;
}

int lgui_box_pack_start(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TBOOLEAN);
	luaL_checktype(L, 3, LUA_TBOOLEAN);
	luaL_checktype(L, 4, LUA_TNUMBER);

	int top = lua_gettop(L), i;
	Object* box = lua_touserdata(L, 1);
	gboolean expand = lua_toboolean(L, 2);
	gboolean fill	= lua_toboolean(L, 3);
	guint padding	= lua_tointeger(L, 4);

	/* Widget by widget */
	for(i = 5; i <= top; i++)
	{
		luaL_checktype(L, i, LUA_TUSERDATA);
		Object* widget = lua_touserdata(L, i);
		gtk_box_pack_start(GTK_BOX(box->pointer), GTK_WIDGET(widget->pointer), expand, fill, padding);
	}

	return 0;
}

int lgui_box_pack_end(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TBOOLEAN);
	luaL_checktype(L, 3, LUA_TBOOLEAN);
	luaL_checktype(L, 4, LUA_TNUMBER);

	int top = lua_gettop(L), i;
	Object* box = lua_touserdata(L, 1);
	gboolean expand = lua_toboolean(L, 2);
	gboolean fill	= lua_toboolean(L, 3);
	guint padding	= lua_tointeger(L, 4);

	/* Widget by widget */
	for(i = 5; i <= top; i++)
	{
		luaL_checktype(L, i, LUA_TUSERDATA);
		Object* widget = lua_touserdata(L, i);
		gtk_box_pack_end(GTK_BOX(box->pointer), GTK_WIDGET(widget->pointer), expand, fill, padding);
	}

	return 0;
}

/* ButtonBox */

int lgui_hbutton_box_new(lua_State* L)
{
	GtkWidget* ptr = gtk_hbutton_box_new();
	object_new(L, ptr, FALSE);
	
	return 1;
}

int lgui_vbutton_box_new(lua_State* L)
{
	GtkWidget* ptr = gtk_vbutton_box_new();
	object_new(L, ptr, FALSE);
	
	return 1;
}
