/*
	This file is part of lgui.

	lgui is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lgui is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lgui.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/

#include "builder.h"

#ifndef G_OS_WIN32
static gboolean priv_initialized = FALSE;

/*
 * Copy from gtkbuilder.c, to solve a problem with the lazy loading.
 */
static GType priv_builder_resolve_type(const gchar *name)
{
	static GModule* module = NULL;
	GType (*func)();
	GString* symbol_name = g_string_new("");
	char c, *symbol;
	int i;
	GType gtype = G_TYPE_INVALID;

	/* Workaround: manually pass the gtk library name*/
	if(!module)
		module = g_module_open(LIB_NAME_GTK, 0);

	for (i = 0; name[i] != '\0'; i++)
	{
		c = name[i];
		/* Skip if uppercase, first or previous is uppercase */
	  	if((c == g_ascii_toupper(c) &&
		   i > 0 && name[i-1] != g_ascii_toupper (name[i-1])) ||
		  (i > 2 && name[i]   == g_ascii_toupper (name[i]) &&
		   name[i-1] == g_ascii_toupper (name[i-1]) &&
		   name[i-2] == g_ascii_toupper (name[i-2])))
		g_string_append_c (symbol_name, '_');
		g_string_append_c (symbol_name, g_ascii_tolower (c));
	}
	
	g_string_append (symbol_name, "_get_type");
	symbol = g_string_free(symbol_name, FALSE);

	if(g_module_symbol(module, symbol, (gpointer)&func))
		gtype = func();

	g_free(symbol);

	return gtype;
}

static GType priv_builder_get_type_from_name(GtkBuilder *builder, const gchar *name)
{
	GType gtype;
	gtype = g_type_from_name(name);
  
  	if(gtype != G_TYPE_INVALID)
    	return gtype;

	return priv_builder_resolve_type(name);
}
#endif

int lgui_builder_new(lua_State* L)
{
	GtkBuilder* ptr =  gtk_builder_new();
	
	#ifndef G_OS_WIN32
	if(!priv_initialized)
	{
		GtkBuilderClass *cl = (GtkBuilderClass*)G_OBJECT_GET_CLASS(ptr);
    	cl->get_type_from_name = priv_builder_get_type_from_name;
    	priv_initialized = TRUE;
	}
	#endif
	
	object_new(L, ptr, TRUE);
	
	return 1;
}

int lgui_builder_add_from_file(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);
	
	GError* errors = NULL;
	
	Object* obj = lua_touserdata(L, 1);
	guint res = gtk_builder_add_from_file(GTK_BUILDER(obj->pointer), lua_tostring(L, 2), &errors);
	
	if(errors)
	{
		g_warning("Error loading GtkBuilder UI definition: %s\n", errors->message);
		g_error_free(errors);
	}
	
	lua_pushinteger(L, res);
	return 1;
}

int lgui_builder_add_from_string(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);
	int top = lua_gettop(L), len = -1;
	
	if(top > 2)
	{
		luaL_checktype(L, 3, LUA_TNUMBER);
		len = lua_tointeger(L, 3);
	}
	
	if(len == -1) len = lua_strlen(L, 2);
	GError* errors = NULL;
	
	Object* obj = lua_touserdata(L, 1);
	guint res = gtk_builder_add_from_string(GTK_BUILDER(obj->pointer), lua_tostring(L, 2), len, &errors);
	
	if(errors)
	{
		g_warning("Error loading GtkBuilder UI definition: %s\n", errors->message);
		g_error_free(errors);
	}	
	
	lua_pushinteger(L, res);
	
	return 1;
}

int lgui_builder_get_object(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);
	
	Object* obj = lua_touserdata(L, 1);
	GObject* ptr = gtk_builder_get_object(GTK_BUILDER(obj->pointer), lua_tostring(L, 2));
	object_new(L, ptr, FALSE);
	
	return 1;
}

int lgui_builder_get_objects(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);

	GSList* list = gtk_builder_get_objects(GTK_BUILDER(obj->pointer));
	lua_pushstring(L, "__lobjGSList2table");
	lua_rawget(L, LUA_GLOBALSINDEX);
	lua_pushlightuserdata(L, list);
	lua_pushinteger(L, TYPE_OBJECT);
	lua_call(L, 2, 1);
	
	/* Free the list (the content will be Garbage-collected) */
	g_slist_free(list);

	return 1;
}
