/*
	This file is part of lgui.

	lgui is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lgui is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lgui.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/

#include "cell.h"

/* GtkCellRenderer */

int lgui_cell_renderer_text_new(lua_State* L)
{
	GtkCellRenderer* ptr = gtk_cell_renderer_text_new();
	object_new(L, ptr, FALSE);

	return 1;
}

int lgui_cell_renderer_pixbuf_new(lua_State* L)
{
	GtkCellRenderer* ptr = gtk_cell_renderer_pixbuf_new();
	object_new(L, ptr, FALSE);

	return 1;
}

int lgui_cell_renderer_progress_new(lua_State* L)
{
	GtkCellRenderer* ptr = gtk_cell_renderer_progress_new();
	object_new(L, ptr, FALSE);

	return 1;
}

int lgui_cell_renderer_toggle_new(lua_State* L)
{
	GtkCellRenderer* ptr = gtk_cell_renderer_toggle_new();
	object_new(L, ptr, FALSE);

	return 1;
}

int lgui_cell_renderer_accel_new(lua_State* L)
{
	GtkCellRenderer* ptr = gtk_cell_renderer_accel_new();
	object_new(L, ptr, FALSE);

	return 1;
}

int lgui_cell_renderer_combo_new(lua_State* L)
{
	GtkCellRenderer* ptr = gtk_cell_renderer_combo_new();
	object_new(L, ptr, FALSE);

	return 1;
}

int lgui_cell_renderer_spin_new(lua_State* L)
{
	GtkCellRenderer* ptr = gtk_cell_renderer_spin_new();
	object_new(L, ptr, FALSE);

	return 1;
}

/* GtkCellLayout */

int lgui_cell_layout_add_attribute(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	int top = lua_gettop(L), i;

	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 2);

	for(i = 3; i < top; i += 2)
	{		
		luaL_checktype(L, i, LUA_TSTRING);
		luaL_checktype(L, i + 1, LUA_TNUMBER);

		gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(obj1->pointer), GTK_CELL_RENDERER(obj2->pointer),
			lua_tostring(L, i), lua_tointeger(L, i + 1)); 
	}

	return 0;
}

int lgui_cell_layout_clear_attributes(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	int top = lua_gettop(L), i;

	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = NULL;
	
	for(i = 2; i < top; ++i)
	{
		luaL_checktype(L, i, LUA_TUSERDATA);
		obj2 = lua_touserdata(L, i);
		gtk_cell_layout_clear_attributes(GTK_CELL_LAYOUT(obj1->pointer), GTK_CELL_RENDERER(obj2->pointer));	
	}	
	
	return 0;
}

int lgui_cell_layout_clear(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);

	gtk_cell_layout_clear(GTK_CELL_LAYOUT(obj->pointer));
	
	return 0;
}

int lgui_cell_layout_pack_start(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	luaL_checktype(L, 3, LUA_TBOOLEAN);
	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 2);

	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(obj1->pointer), GTK_CELL_RENDERER(obj2->pointer), lua_toboolean(L, 3));

	return 0;
}

int lgui_cell_layout_pack_end(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 2);

	gtk_cell_layout_pack_end(GTK_CELL_LAYOUT(obj1->pointer), GTK_CELL_RENDERER(obj2->pointer), lua_toboolean(L, 3));

	return 0;
}

int lgui_cell_layout_reorder(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	luaL_checktype(L, 3, LUA_TNUMBER);

	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 2);

	gtk_cell_layout_reorder(GTK_CELL_LAYOUT(obj1->pointer), GTK_CELL_RENDERER(obj2->pointer), lua_tointeger(L, 3));
	
	return 0;
}

