/*
	This file is part of lgui.

	lgui is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lgui is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lgui.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/

#include "combobox.h"

/* GtkComboBox */

int lgui_combo_box_new(lua_State* L)
{
	GtkWidget* ptr = gtk_combo_box_new();
	object_new(L, ptr, FALSE);

	return 1;
}

int lgui_combo_box_new_with_model(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);

	GtkWidget* ptr = gtk_combo_box_new_with_model(GTK_TREE_MODEL(obj->pointer));
	object_new(L, ptr, FALSE);

	return 1;
}

int lgui_combo_box_get_active_iter(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 2);

	gboolean res = gtk_combo_box_get_active_iter(GTK_COMBO_BOX(obj1->pointer),
		(GtkTreeIter*)obj2->pointer);

	lua_pushboolean(L, res);
	return 1;
}

int lgui_combo_box_set_model(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 2);

	gtk_combo_box_set_model(GTK_COMBO_BOX(obj1->pointer), GTK_TREE_MODEL(obj2->pointer));
	return 0;
}

int lgui_combo_box_set_active(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);
	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 2);

	gtk_combo_box_set_model(GTK_COMBO_BOX(obj1->pointer), GTK_TREE_MODEL(obj2->pointer));
	return 0;
}

/* Simplified - Text only */

int lgui_combo_box_new_text(lua_State* L)
{
	GtkWidget* ptr = gtk_combo_box_new_text();
	object_new(L, ptr, FALSE);

	return 1;
}

int lgui_combo_box_append_text(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	int top = lua_gettop(L), i;
	Object* obj = lua_touserdata(L, 1);
	
	for(i = 2; i <= top; ++i)
	{
		luaL_checktype(L, i, LUA_TSTRING);
		gtk_combo_box_append_text(GTK_COMBO_BOX(obj->pointer), lua_tostring(L, i));
	}	

	return 0;
}

int lgui_combo_box_prepend_text(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	int top = lua_gettop(L), i;
	Object* obj = lua_touserdata(L, 1);
	
	for(i = 2; i <= top; ++i)
	{
		luaL_checktype(L, i, LUA_TSTRING);
		gtk_combo_box_prepend_text(GTK_COMBO_BOX(obj->pointer), lua_tostring(L, i));
	}	

	return 0;
}

int lgui_combo_box_get_active_text(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);

	Object* obj = lua_touserdata(L, 1);
	gchar* res = gtk_combo_box_get_active_text(GTK_COMBO_BOX(obj->pointer));
	lua_pushstring(L, res);
	g_free(res);

	return 1;
}

int lgui_combo_box_remove_text(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);

	Object* obj = lua_touserdata(L, 1);
	gtk_combo_box_remove_text(GTK_COMBO_BOX(obj->pointer), lua_tonumber(L, 2));

	return 0;
}

int lgui_combo_box_insert_text(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);
	luaL_checktype(L, 3, LUA_TSTRING);

	Object* obj = lua_touserdata(L, 1);
	gtk_combo_box_insert_text(GTK_COMBO_BOX(obj->pointer), lua_tonumber(L, 2), lua_tostring(L, 3));

	return 0;
}

/* GtkComboBoxEntry */

int lgui_combo_box_entry_new(lua_State* L)
{
	GtkWidget* ptr = gtk_combo_box_entry_new();
	object_new(L, ptr, FALSE);
	
	return 1;
}

int lgui_combo_box_entry_new_text(lua_State* L)
{
	GtkWidget* ptr = gtk_combo_box_entry_new_text();
	object_new(L, ptr, FALSE);
	
	return 1;
}

int lgui_combo_box_entry_new_with_model(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);
	Object* obj = lua_touserdata(L, 1);
	
	GtkWidget* ptr = gtk_combo_box_entry_new_with_model(GTK_TREE_MODEL(obj->pointer),
		lua_tointeger(L, 2));
	object_new(L, ptr, FALSE);
	
	return 1;
}
