/*
	This file is part of lgui.

	lgui is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lgui is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lgui.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/

#include "editable.h"

int lgui_editable_insert_text(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);
	luaL_checktype(L, 3, LUA_TNUMBER);
	
	Object* obj = lua_touserdata(L, 1);
	const gchar* text = lua_tostring(L, 2);
	gint pos = lua_tointeger(L, 3);
	gint len = -1;
	
	gtk_editable_insert_text(GTK_EDITABLE(obj->pointer), text, len, &pos);
	lua_pushnumber(L, pos);
	
	return 1;
}

int lgui_editable_delete_text(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);
	luaL_checktype(L, 3, LUA_TNUMBER);
	
	Object* obj = lua_touserdata(L, 1);
	gtk_editable_delete_text(GTK_EDITABLE(obj->pointer), lua_tointeger(L, 2),
		lua_tointeger(L, 3));
	
	return 0;
}

int lgui_editable_get_chars(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);
	luaL_checktype(L, 3, LUA_TNUMBER);
	
	Object* obj = lua_touserdata(L, 1);
	gchar* text = gtk_editable_get_chars(GTK_EDITABLE(obj->pointer),
		lua_tointeger(L, 2), lua_tointeger(L, 3));
	lua_pushstring(L, text);
	g_free(text);
	
	return 1;
}

int lgui_editable_select_region(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);
	luaL_checktype(L, 3, LUA_TNUMBER);
	
	Object* obj = lua_touserdata(L, 1);
	gtk_editable_select_region(GTK_EDITABLE(obj->pointer), lua_tointeger(L, 2),
		lua_tointeger(L, 3));
	
	return 0;
}

int lgui_editable_get_selection_bounds(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	
	Object* obj = lua_touserdata(L, 1);
	gint pos1, pos2;
	
	gtk_editable_get_selection_bounds(GTK_EDITABLE(obj->pointer), &pos1, &pos2);
	lua_pushinteger(L, pos1);
	lua_pushinteger(L, pos2);
	
	return 2;
}

int lgui_editable_cut_clipboard(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	gtk_editable_cut_clipboard(GTK_EDITABLE(obj->pointer));
	
	return 0;
}

int lgui_editable_copy_clipboard(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	gtk_editable_copy_clipboard(GTK_EDITABLE(obj->pointer));
	
	return 0;
}

int lgui_editable_paste_clipboard(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	gtk_editable_paste_clipboard(GTK_EDITABLE(obj->pointer));
	
	return 0;
}

int lgui_editable_delete_selection(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	gtk_editable_delete_selection(GTK_EDITABLE(obj->pointer));
	
	return 0;
}

int lgui_editable_set_position(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);
	Object* obj = lua_touserdata(L, 1);
	gtk_editable_set_position(GTK_EDITABLE(obj->pointer), lua_tointeger(L, 2));
	
	return 0;
}

int lgui_editable_get_position(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	gint pos = gtk_editable_get_position(GTK_EDITABLE(obj->pointer));
	lua_pushinteger(L, pos);
	
	return 1;
}
