/*
	This file is part of lgui.

	lgui is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lgui is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lgui.  If not, see <http://www.gnu.org/licenses/>.

    Copyright (C) 2008 Lucas Hermann Negri
*/

#include "lgui.h"

int lgui_main(lua_State* L)
{
	gtk_main();
	return 0;
}

int lgui_quit(lua_State* L)
{
	gtk_main_quit();
	return 0;
}

int lgui_events_pending(lua_State* L)
{
	lua_pushboolean(L, gtk_events_pending());
	return 1;
}

int lgui_main_iteration(lua_State* L)
{
	lua_pushboolean(L, gtk_main_iteration());
	return 1;
}

int lgui_version(lua_State* L)
{
	char rv[21], cv[21];

	snprintf(rv, 20, "%d.%d.%d", gtk_major_version, gtk_minor_version, gtk_micro_version);
	snprintf(cv, 20, "%d.%d.%d", GTK_MAJOR_VERSION, GTK_MINOR_VERSION, GTK_MICRO_VERSION);

	lua_pushstring(L, LIB_VERSION);
	lua_pushstring(L, rv);
	lua_pushstring(L, cv);

	return 3;
}

gboolean object_new(lua_State* L, gpointer ptr, gboolean need_unref)
{
	lua_pushstring(L, "__lobjObjectNew");
	lua_rawget(L, LUA_GLOBALSINDEX);
	lua_pushlightuserdata(L, ptr);
	lua_pushboolean(L, need_unref);
	lua_call(L, 2, 1);

	/* Now configue the metatable */
	if(lua_isuserdata(L, -1))
	{
		int top = lua_gettop(L);
		Object* obj = lua_touserdata(L, -1);
		const gchar* type = g_type_name(G_TYPE_FROM_INSTANCE(obj->pointer));
		lua_getmetatable(L, -1);

		/* Set the index */
		lua_pushstring(L, "__index");
		lua_pushstring(L, "__lguiGetClassTable");
		lua_rawget(L, LUA_GLOBALSINDEX);
		lua_pushstring(L, type);
		lua_call(L, 1, 1);

		lua_rawset(L, -3);

		/* The userdata must be at the top */
		lua_settop(L, top);
		return TRUE;
	}

	return FALSE;
}

gboolean struct_new(lua_State* L, gpointer ptr, gboolean need_unref)
{
	lua_pushstring(L, "__lobjStructNew");
	lua_rawget(L, LUA_GLOBALSINDEX);
	lua_pushlightuserdata(L, ptr);
	lua_pushboolean(L, need_unref);
	lua_call(L, 2, 1);

	return TRUE;
}

void callback_free(gpointer user_data)
{
	Data* data = (Data*)user_data;

    /* Remove refs from Lua side */
    luaL_unref(data->L, LUA_REGISTRYINDEX, data->function_ref);
    luaL_unref(data->L, LUA_REGISTRYINDEX, data->ud_ref);

    /* Free the data */
    g_free(data);

    #ifdef IDEBUG
	g_debug("Released Callback data!");
	#endif
}
