/*
	This file is part of lgui.

	lgui is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lgui is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lgui.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/

#include "pixbuf.h"

int lgui_pixbuf_new_from_file(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TSTRING);
	GError* errors = NULL;
	GdkPixbuf* ptr = gdk_pixbuf_new_from_file(lua_tostring(L, 1), &errors);

	if(errors)
	{
		lua_pushboolean(L, FALSE);
		lua_pushstring(L, errors->message);
		g_error_free(errors);
		if(G_IS_OBJECT(ptr)) g_object_unref(ptr);
		return 2;
	}

	object_new(L, ptr, TRUE);
	return 1;
}

int lgui_pixbuf_new_from_file_at_scale(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TSTRING);
	luaL_checktype(L, 2, LUA_TNUMBER);
	luaL_checktype(L, 3, LUA_TNUMBER);
	luaL_checktype(L, 4, LUA_TBOOLEAN);

	GError* errors = NULL;
	GdkPixbuf* ptr = gdk_pixbuf_new_from_file_at_scale(lua_tostring(L, 1),
	lua_tointeger(L, 2), lua_tointeger(L, 3), lua_toboolean(L, 4), &errors);

	if(errors)
	{
		lua_pushboolean(L, FALSE);
		lua_pushstring(L, errors->message);
		g_error_free(errors);
		if(G_IS_OBJECT(ptr)) g_object_unref(ptr);
		return 2;
	}

	object_new(L, ptr, TRUE);
	return 1;
}

int lgui_pixbuf_save(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);
	luaL_checktype(L, 3, LUA_TSTRING);

	GError* errors = NULL;
	Object* obj = lua_touserdata(L, 1);
	gboolean res = gdk_pixbuf_save(GDK_PIXBUF(obj->pointer), lua_tostring(L, 2),
	lua_tostring(L, 3), &errors, NULL);
	lua_pushboolean(L, res);

	if(errors)
	{
		lua_pushstring(L, errors->message);
		g_error_free(errors);
		return 2;
	}

	return 1;
}

int lgui_pixbuf_scale_simple(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);
	luaL_checktype(L, 3, LUA_TNUMBER);
	luaL_checktype(L, 4, LUA_TNUMBER);
	Object* obj = lua_touserdata(L, 1);
	
	GdkPixbuf* ptr = gdk_pixbuf_scale_simple(GDK_PIXBUF(obj->pointer),
		lua_tointeger(L, 2), lua_tointeger(L, 3), (GdkInterpType)lua_tointeger(L, 4));
	object_new(L, ptr, TRUE);
	 
	return 1;
}

int lgui_pixbuf_rotate_simple(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);
	Object* obj = lua_touserdata(L, 1);
	
	GdkPixbuf* ptr = gdk_pixbuf_rotate_simple(GDK_PIXBUF(obj->pointer), 
		(GdkPixbufRotation)lua_tointeger(L, 2));
	object_new(L, ptr, TRUE);
	
	return 1;
}
