/*
	This file is part of lgui.

	lgui is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lgui is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lgui.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/

#include "text.h"

/* GtkTextView */

int lgui_text_view_new(lua_State* L)
{
	GtkWidget* ptr = gtk_text_view_new();
	object_new(L, ptr, FALSE);

	return 1;
}

int lgui_text_view_new_with_buffer(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	GtkWidget* ptr = gtk_text_view_new_with_buffer(GTK_TEXT_BUFFER(obj->pointer));
	object_new(L, ptr, FALSE);

	return 1;
}

int lgui_text_view_scroll_to_iter(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	
	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 2);
	int top = lua_gettop(L);
	gboolean res = FALSE;
	
	if(top == 2)
	{
		res = gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(obj1->pointer),
			(GtkTextIter*)obj2->pointer, 0, FALSE, 0, 0);
	}
	else
	{	
		luaL_checktype(L, 3, LUA_TNUMBER);
		luaL_checktype(L, 4, LUA_TBOOLEAN);
		luaL_checktype(L, 5, LUA_TNUMBER);
		luaL_checktype(L, 6, LUA_TNUMBER);
	
		res = gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(obj1->pointer),
			(GtkTextIter*)obj2->pointer, lua_tonumber(L, 3), lua_toboolean(L, 4),
			lua_tonumber(L, 5), lua_tonumber(L, 6));
	}
	
	lua_pushboolean(L, res);
	return 1;
}

int lgui_text_view_scroll_to_mark(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	
	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 2);
	int top = lua_gettop(L);
	
	if(top == 2)
	{
		gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(obj1->pointer),
			(GtkTextMark*)obj2->pointer, 0, FALSE, 0, 0);
	}
	else
	{	
		luaL_checktype(L, 3, LUA_TNUMBER);
		luaL_checktype(L, 4, LUA_TBOOLEAN);
		luaL_checktype(L, 5, LUA_TNUMBER);
		luaL_checktype(L, 6, LUA_TNUMBER);
	
		gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(obj1->pointer),
			(GtkTextMark*)obj2->pointer, lua_tonumber(L, 3), lua_toboolean(L, 4),
			lua_tonumber(L, 5), lua_tonumber(L, 6));
	}
	
	return 0;
}

/* GtkTextTagTable */

int lgui_text_tag_table_add(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 2);
	
	gtk_text_tag_table_add(GTK_TEXT_TAG_TABLE(obj1->pointer), GTK_TEXT_TAG(obj2->pointer)); 
	
	return 0;
}

int lgui_text_tag_table_remove(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 2);
	
	gtk_text_tag_table_remove(GTK_TEXT_TAG_TABLE(obj1->pointer), GTK_TEXT_TAG(obj2->pointer)); 
	
	return 0;
}

int lgui_text_tag_table_lookup(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);
	Object* obj = lua_touserdata(L, 1);
	
	GtkTextTag* ptr = gtk_text_tag_table_lookup(GTK_TEXT_TAG_TABLE(obj->pointer), lua_tostring(L, 2)); 
	object_new(L, ptr, FALSE);
	
	return 1;
}

int lgui_text_tag_table_get_size(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	gint res = gtk_text_tag_table_get_size(GTK_TEXT_TAG_TABLE(obj->pointer)); 
	lua_pushnumber(L, res);
	
	return 1;
}

/* GtkTextBuffer */

int lgui_text_buffer_new(lua_State* L)
{
	int top = lua_gettop(L);
	GtkTextTagTable* table = NULL;
	
	if(top > 0)
	{
		luaL_checktype(L, 1, LUA_TUSERDATA);
		Object* obj = lua_touserdata(L, 1);
		table = obj->pointer;
	}
	
	GtkTextBuffer* ptr = gtk_text_buffer_new(table);
	object_new(L, ptr, TRUE);
	
	return 1;
}

int lgui_text_buffer_get_line_count(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	gint lines = gtk_text_buffer_get_line_count(GTK_TEXT_BUFFER(obj->pointer));
	lua_pushnumber(L, lines);

	return 1;
}

int lgui_text_buffer_get_selection_bounds(lua_State* L)
{
	int top = lua_gettop(L);
	
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj1 = lua_touserdata(L, 1);
	GtkTextIter *iter1 = NULL, *iter2 = NULL;
	
	if(top > 2)
	{
		luaL_checktype(L, 2, LUA_TUSERDATA);
		luaL_checktype(L, 3, LUA_TUSERDATA);
	
		Object* obj2 = lua_touserdata(L, 2);
		Object* obj3 = lua_touserdata(L, 3);
		iter1 = obj2->pointer;
		iter2 = obj3->pointer;
	}
	
	gboolean res = gtk_text_buffer_get_selection_bounds(
		GTK_TEXT_BUFFER(obj1->pointer), iter1, iter2);
	
	lua_pushboolean(L, res);
	
	return 1;
}

int lgui_text_buffer_get_char_count(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	gint lines = gtk_text_buffer_get_char_count(GTK_TEXT_BUFFER(obj->pointer));
	lua_pushnumber(L, lines);

	return 1;
}

int lgui_text_buffer_insert(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	luaL_checktype(L, 3, LUA_TSTRING);

	gint len = -1;

	/* Passed a length */
	if(lua_gettop(L) >= 4)
	{
		luaL_checktype(L, 4, LUA_TNUMBER);
		len = lua_tointeger(L, 4);
	}

	Object* obj = lua_touserdata(L, 1);
	Object* stc = lua_touserdata(L, 2);

	gtk_text_buffer_insert(GTK_TEXT_BUFFER(obj->pointer), (GtkTextIter*)stc->pointer,
	lua_tostring(L, 3), len);

	return 0;
}

int lgui_text_buffer_insert_with_tags(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	luaL_checktype(L, 3, LUA_TSTRING);
	luaL_checktype(L, 4, LUA_TNUMBER);

	Object* obj = lua_touserdata(L, 1);
	Object* stc = lua_touserdata(L, 2);

	/* Get the start offset */
	gint len = lua_tointeger(L, 4), i;
	gint start_offset = gtk_text_iter_get_offset((GtkTextIter*)stc->pointer);

	/* Insert the text */
	gtk_text_buffer_insert(GTK_TEXT_BUFFER(obj->pointer), (GtkTextIter*)stc->pointer,
	lua_tostring(L, 3), len);

	/* Get the initial iterator */
	GtkTextIter start;
	gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(obj->pointer), &start, start_offset);

	/* Apply the tags */
	for(i = 5; i <= lua_gettop(L); ++i)
	{
		Object* tag = lua_touserdata(L, i);
		gtk_text_buffer_apply_tag(GTK_TEXT_BUFFER(obj->pointer), GTK_TEXT_TAG(tag->pointer),
		&start, (GtkTextIter*)stc->pointer);
	}

	return 0;
}

int lgui_text_buffer_insert_pixbuf(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	luaL_checktype(L, 3, LUA_TUSERDATA);


	Object* obj = lua_touserdata(L, 1);
	Object* stc = lua_touserdata(L, 2);
	Object* buf = lua_touserdata(L, 3);

	gtk_text_buffer_insert_pixbuf(GTK_TEXT_BUFFER(obj->pointer), (GtkTextIter*)stc->pointer,
	GDK_PIXBUF(buf->pointer));

	return 0;
}

int lgui_text_buffer_get_start_iter(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	Object* stc = lua_touserdata(L, 2);
	gtk_text_buffer_get_start_iter(GTK_TEXT_BUFFER(obj->pointer), (GtkTextIter*)stc->pointer);

	return 0;
}

int lgui_text_buffer_get_end_iter(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	Object* stc = lua_touserdata(L, 2);
	gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(obj->pointer), (GtkTextIter*)stc->pointer);

	return 0;
}

int lgui_text_buffer_get_iter_at_offset(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	luaL_checktype(L, 3, LUA_TNUMBER);

	Object* obj = lua_touserdata(L, 1);
	Object* stc = lua_touserdata(L, 2);
	gtk_text_buffer_get_iter_at_offset(GTK_TEXT_BUFFER(obj->pointer), (GtkTextIter*)stc->pointer,
	lua_tointeger(L, 3));

	return 0;
}

int lgui_text_buffer_get_iter_at_line(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	luaL_checktype(L, 3, LUA_TNUMBER);

	Object* obj = lua_touserdata(L, 1);
	Object* stc = lua_touserdata(L, 2);
	gtk_text_buffer_get_iter_at_line(GTK_TEXT_BUFFER(obj->pointer), (GtkTextIter*)stc->pointer,
	lua_tointeger(L, 3));

	return 0;
}

int lgui_text_buffer_get_iter_at_line_offset(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	luaL_checktype(L, 3, LUA_TNUMBER);
	luaL_checktype(L, 4, LUA_TNUMBER);

	Object* obj = lua_touserdata(L, 1);
	Object* stc = lua_touserdata(L, 2);
	gtk_text_buffer_get_iter_at_line_offset(GTK_TEXT_BUFFER(obj->pointer), (GtkTextIter*)stc->pointer,
	lua_tointeger(L, 3), lua_tointeger(L, 4));

	return 0;
}

int lgui_text_buffer_create_tag(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);
	Object* obj = lua_touserdata(L, 1);

	GtkTextTag* ptr = gtk_text_buffer_create_tag(GTK_TEXT_BUFFER(obj->pointer), lua_tostring(L, 2), NULL);
	object_new(L, ptr, FALSE);

	return 1;
}

int lgui_text_buffer_select_range(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	luaL_checktype(L, 3, LUA_TUSERDATA);
	
	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 2);
	Object* obj3 = lua_touserdata(L, 3);
	
	gtk_text_buffer_select_range(GTK_TEXT_BUFFER(obj1->pointer),
		(GtkTextIter*)obj2->pointer, (GtkTextIter*)obj3->pointer);
	
	return 0;
}

int lgui_text_buffer_place_cursor(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	
	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 2);
	
	gtk_text_buffer_place_cursor(GTK_TEXT_BUFFER(obj1->pointer), (GtkTextIter*)obj2->pointer);
	
	return 0;
}

int lgui_text_buffer_delete_selection(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TBOOLEAN);
	luaL_checktype(L, 3, LUA_TBOOLEAN);
	Object* obj = lua_touserdata(L, 1);
	
	gboolean res = gtk_text_buffer_delete_selection(GTK_TEXT_BUFFER(obj->pointer),	
		lua_toboolean(L, 2), lua_toboolean(L, 3));
	lua_pushboolean(L, res);
	
	return 1;
}

int lgui_text_buffer_copy_clipboard(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	
	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 2);
	gtk_text_buffer_copy_clipboard(GTK_TEXT_BUFFER(obj1->pointer), 
		GTK_CLIPBOARD(obj2->pointer));
		
	return 0;
}

int lgui_text_buffer_cut_clipboard(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	luaL_checktype(L, 3, LUA_TBOOLEAN);
	
	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 2);
	gtk_text_buffer_cut_clipboard(GTK_TEXT_BUFFER(obj1->pointer), 
		GTK_CLIPBOARD(obj2->pointer), lua_toboolean(L, 3));
		
	return 0;
}

int lgui_text_buffer_paste_clipboard(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	luaL_checktype(L, 4, LUA_TBOOLEAN);
	
	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 2);
	
	GtkTextIter* iter = NULL;
	
	if(lua_isuserdata(L, 3))
		iter = (GtkTextIter*)((Object*)lua_touserdata(L, 3))->pointer;
	
	gtk_text_buffer_paste_clipboard(GTK_TEXT_BUFFER(obj1->pointer), 
		GTK_CLIPBOARD(obj2->pointer), iter, lua_toboolean(L, 4));
		
	return 0;
}

/* GtkTextIter */

int lgui_text_iter_new(lua_State* L)
{
	GtkTextIter* ptr = g_malloc(sizeof(GtkTextIter));
	struct_new(L, ptr, TRUE);

	return 1;
}

int lgui_text_iter_set_offset(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);
	
	Object* obj = lua_touserdata(L, 1);
	gtk_text_iter_set_offset((GtkTextIter*)obj->pointer, lua_tointeger(L, 2));
	
	return 0;
}

int lgui_text_iter_get_offset(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	
	Object* obj = lua_touserdata(L, 1);
	lua_pushnumber(L, gtk_text_iter_get_offset((GtkTextIter*)obj->pointer));
	
	return 1;
}

int lgui_text_iter_set_line(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);
	
	Object* obj = lua_touserdata(L, 1);
	gtk_text_iter_set_line((GtkTextIter*)obj->pointer, lua_tointeger(L, 2));
	
	return 0;
}

int lgui_text_iter_get_line(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	
	Object* obj = lua_touserdata(L, 1);
	lua_pushnumber(L, gtk_text_iter_get_offset((GtkTextIter*)obj->pointer));
	
	return 1;
}

int lgui_text_iter_forward_search(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);
	luaL_checktype(L, 3, LUA_TNUMBER);
	int top = lua_gettop(L);
	
	GtkTextIter* iter  = (GtkTextIter*)((Object*)lua_touserdata(L, 1))->pointer;
	GtkTextIter* start = top > 3 ? (GtkTextIter*)((Object*)lua_touserdata(L, 4))->pointer : NULL;
	GtkTextIter* stop = top > 4 ? (GtkTextIter*)((Object*)lua_touserdata(L, 5))->pointer : NULL;
	GtkTextIter* limit = top > 5 ? (GtkTextIter*)((Object*)lua_touserdata(L, 6))->pointer : NULL;
	
	lua_pushboolean(L, gtk_text_iter_forward_search(iter, lua_tostring(L, 2),
		lua_tointeger(L, 3), start, stop, limit));
		
	return 1;
}

int lgui_text_iter_backward_search(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);
	luaL_checktype(L, 3, LUA_TNUMBER);
	int top = lua_gettop(L);
	
	GtkTextIter* iter  = (GtkTextIter*)((Object*)lua_touserdata(L, 1))->pointer;
	GtkTextIter* start = top > 3 ? (GtkTextIter*)((Object*)lua_touserdata(L, 4))->pointer : NULL;
	GtkTextIter* stop = top > 4 ? (GtkTextIter*)((Object*)lua_touserdata(L, 5))->pointer : NULL;
	GtkTextIter* limit = top > 5 ? (GtkTextIter*)((Object*)lua_touserdata(L, 6))->pointer : NULL;
	
	lua_pushboolean(L, gtk_text_iter_backward_search(iter, lua_tostring(L, 2),
		lua_tointeger(L, 3), start, stop, limit));
		
	return 1;
}

int lgui_text_buffer_get_iter_at_mark(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	luaL_checktype(L, 3, LUA_TUSERDATA);
	
	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 2);
	Object* obj3 = lua_touserdata(L, 3);
	
	gtk_text_buffer_get_iter_at_mark(GTK_TEXT_BUFFER(obj1->pointer),
		(GtkTextIter*)obj2->pointer, (GtkTextMark*)obj3->pointer);
		
	return 0;
}

int lgui_text_buffer_create_mark(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);
	luaL_checktype(L, 3, LUA_TUSERDATA);
	luaL_checktype(L, 4, LUA_TBOOLEAN);
	
	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 3);
	
	GtkTextMark* mark = gtk_text_buffer_create_mark(GTK_TEXT_BUFFER(obj1->pointer), lua_tostring(L, 2),
		(GtkTextIter*)obj2->pointer, lua_toboolean(L, 4));
		
	object_new(L, mark, FALSE);
	return 1;
}

int lgui_text_buffer_move_mark(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	luaL_checktype(L, 3, LUA_TUSERDATA);
	
	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 2);
	Object* obj3 = lua_touserdata(L, 3);
	
	gtk_text_buffer_move_mark(GTK_TEXT_BUFFER(obj1->pointer),
		GTK_TEXT_MARK(obj2->pointer), (GtkTextIter*)obj3->pointer);
		
	return 0;
}

int lgui_text_buffer_delete_mark(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	
	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 2);
	
	gtk_text_buffer_delete_mark(GTK_TEXT_BUFFER(obj1->pointer),
		GTK_TEXT_MARK(obj2->pointer));
		
	return 0;
}

int lgui_text_buffer_delete(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	luaL_checktype(L, 3, LUA_TUSERDATA);
	
	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 2);
	Object* obj3 = lua_touserdata(L, 3);
	
	gtk_text_buffer_delete(GTK_TEXT_BUFFER(obj1->pointer),
		(GtkTextIter*)obj2->pointer, (GtkTextIter*)obj3->pointer);
	
	return 0;
}

int lgui_text_buffer_get_mark(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TSTRING);
	
	Object* obj = lua_touserdata(L, 1);
	GtkTextMark* mark = gtk_text_buffer_get_mark(GTK_TEXT_BUFFER(obj->pointer),
		lua_tostring(L, 2));
	object_new(L, mark, FALSE);
	
	return 1;
}
