/*
	This file is part of lgui.

	lgui is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lgui is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lgui.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/

#include "treemodel.h"

int lgui_tree_iter_new(lua_State* L)
{
	GtkTreeIter* ptr = g_malloc(sizeof(GtkTreeIter));
	struct_new(L, ptr, TRUE);

	return 1;
}

int lgui_tree_model_get_iter_from_string(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	luaL_checktype(L, 3, LUA_TSTRING);

	Object* obj = lua_touserdata(L, 1);
	Object* stc = lua_touserdata(L, 2);

	gboolean res = gtk_tree_model_get_iter_from_string(GTK_TREE_MODEL(obj->pointer),
	(GtkTreeIter*)stc->pointer, lua_tostring(L, 3));

	lua_pushboolean(L, res);

	return 1;
}

int lgui_tree_model_get_string_from_iter(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);

	Object* obj = lua_touserdata(L, 1);
	Object* stc = lua_touserdata(L, 2);
	gchar* str = gtk_tree_model_get_string_from_iter(GTK_TREE_MODEL(obj->pointer),
	(GtkTreeIter*)stc->pointer);
	lua_pushstring(L, str);
	g_free(str);

	return 1;
}

int lgui_tree_model_get_iter(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	luaL_checktype(L, 3, LUA_TUSERDATA);

	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 2);
	Object* obj3 = lua_touserdata(L, 3);

	gboolean res = gtk_tree_model_get_iter(GTK_TREE_MODEL(obj1->pointer), (GtkTreeIter*)obj2->pointer,
		(GtkTreePath*)obj3->pointer);

	lua_pushboolean(L, res);

	return 1;
}

int lgui_tree_model_get_iter_first(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);

	Object* obj = lua_touserdata(L, 1);
	Object* stc = lua_touserdata(L, 2);

	gboolean res = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(obj->pointer),
	(GtkTreeIter*)stc->pointer);

	lua_pushboolean(L, res);

	return 1;
}

int lgui_tree_model_iter_next(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);

	Object* obj = lua_touserdata(L, 1);
	Object* stc = lua_touserdata(L, 2);

	gboolean res = gtk_tree_model_iter_next(GTK_TREE_MODEL(obj->pointer),
	(GtkTreeIter*)stc->pointer);

	lua_pushboolean(L, res);

	return 1;
}

int lgui_tree_model_get(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);

	int size = lua_gettop(L), i;
	GtkTreeModel* model = GTK_TREE_MODEL(((Object*)lua_touserdata(L, 1))->pointer);
	GtkTreeIter*  iter  = (GtkTreeIter*)((Object*)lua_touserdata(L, 2))->pointer;
	lua_getfield(L, LUA_GLOBALSINDEX, "__lobjValuePush");

	/* Get field by field */
	for(i = 3; i <= size; ++i)
	{
		luaL_checktype(L, i, LUA_TNUMBER);
		int column = lua_tointeger(L, i);

		#ifdef IDEBUG
		GType type = gtk_tree_model_get_column_type(model, column);
		g_debug("Getting listModel column %i, of type %s!", column, g_type_name(type));
		#endif
		
		GValue val = {0};
		gtk_tree_model_get_value(model, iter, column, &val);
		lua_pushvalue(L, size + 1);
		lua_pushlightuserdata(L, &val);
		lua_call(L, 1, 1);
		g_value_unset(&val);
	}

	return size - 2;
}

int lgui_tree_model_get_n_columns(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);

	Object* obj = lua_touserdata(L, 1);
	gint cols = gtk_tree_model_get_n_columns(GTK_TREE_MODEL(obj->pointer));
	lua_pushnumber(L, cols);

	return 1;
}
