/*
	This file is part of lgui.

	lgui is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lgui is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lgui.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/


#include "treeselection.h"

int lgui_tree_path_handler(lua_State* L)
{
	#ifdef IDEBUG
	g_debug("Handling a GtkTreePath!");
	#endif
	
	GtkTreePath* path = lua_touserdata(L, -1);
	gchar* str = gtk_tree_path_to_string(path);
	lua_pushstring(L, str);
	g_free(str);
	
	return 1;
}

int lgui_tree_selection_set_mode(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);

	Object* obj = lua_touserdata(L, 1);
	gtk_tree_selection_set_mode(GTK_TREE_SELECTION(obj->pointer), (GtkSelectionMode)lua_tointeger(L, 2));

	return 0;
}

int lgui_tree_selection_get_mode(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);

	Object* obj = lua_touserdata(L, 1);
	lua_pushinteger(L, (int)gtk_tree_selection_get_mode(GTK_TREE_SELECTION(obj->pointer)));

	return 1;
}

int lgui_tree_selection_count_selected_rows (lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);

	Object* obj = lua_touserdata(L, 1);
	lua_pushinteger(L, (int)gtk_tree_selection_count_selected_rows(GTK_TREE_SELECTION(obj->pointer)));

	return 1;
}

int lgui_tree_selection_select_iter(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);

	Object* obj = lua_touserdata(L, 1);
	Object* stc = lua_touserdata(L, 2);
	gtk_tree_selection_select_iter(GTK_TREE_SELECTION(obj->pointer), (GtkTreeIter*)stc->pointer);

	return 0;
}

int lgui_tree_selection_unselect_iter(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);

	Object* obj = lua_touserdata(L, 1);
	Object* stc = lua_touserdata(L, 2);
	gtk_tree_selection_unselect_iter(GTK_TREE_SELECTION(obj->pointer), (GtkTreeIter*)stc->pointer);

	return 0;
}

int lgui_tree_selection_iter_is_selected(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);

	Object* obj = lua_touserdata(L, 1);
	Object* stc = lua_touserdata(L, 2);
	gboolean res = gtk_tree_selection_iter_is_selected(GTK_TREE_SELECTION(obj->pointer), (GtkTreeIter*)stc->pointer);
	lua_pushboolean(L, res);

	return 1;
}

int lgui_tree_selection_select_range(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	luaL_checktype(L, 3, LUA_TUSERDATA);

	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 2);
	Object* obj3 = lua_touserdata(L, 3);
	gtk_tree_selection_select_range(GTK_TREE_SELECTION(obj1->pointer),
		(GtkTreePath*)obj2->pointer, (GtkTreePath*)obj3->pointer);

	return 0;
}

int lgui_tree_selection_unselect_range(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	luaL_checktype(L, 3, LUA_TUSERDATA);

	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 2);
	Object* obj3 = lua_touserdata(L, 3);
	gtk_tree_selection_unselect_range(GTK_TREE_SELECTION(obj1->pointer),
		(GtkTreePath*)obj2->pointer, (GtkTreePath*)obj3->pointer);

	return 0;
}

int lgui_tree_selection_get_selected(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);

	Object* obj = lua_touserdata(L, 1);
	Object* iter = lua_touserdata(L, 2);

	GtkTreeModel* tm = NULL;

	gboolean res = gtk_tree_selection_get_selected(GTK_TREE_SELECTION(obj->pointer), &tm, (GtkTreeIter*)iter->pointer);
	lua_pushboolean(L, res);
	object_new(L, tm, FALSE);

	return 2;
}

int lgui_tree_selection_get_selected_rows(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);

	Object* obj = lua_touserdata(L, 1);

	GtkTreeModel* tm = NULL; /* Always NULL. Why? */
	GList* list = gtk_tree_selection_get_selected_rows(GTK_TREE_SELECTION(obj->pointer), &tm);

	lua_pushstring(L, "__lobjGList2table");
	lua_rawget(L, LUA_GLOBALSINDEX);
	lua_pushlightuserdata(L, list);
	lua_pushinteger(L, TYPE_CUSTOM);
	lua_pushcfunction(L, lgui_tree_path_handler);
	lua_call(L, 3, 1);
	
	/* Free the list */
	g_list_foreach(list, (GFunc)gtk_tree_path_free, NULL);
	g_list_free(list);
	
	return 1;
}
